@@ -104,9 +104,11 @@ def parse_offset( term ):
         assert 1 <= len( hms ) <= 3, "only h:mm:ss.s allowed"
         while hms[0] == '':
             hms		= hms[1:] # <:02.5 is OK
-        offset		= reduce( lambda ts, v: ts * 60 + float( v ), [ 0 ] + hms )
+        offset		= 0
+        for v in hms:
+            offset	= offset * 60 + float( v )
         if term[sign] == '<':
-            offset = -offset
+            offset	= -offset
     except Exception as exc:
         raise ValueError( "Invalid offset %r; must be </>[[h:]m:]s[.s]: %s" % ( term, exc ))
     return offset
@@ -136,21 +138,22 @@ class timestamp( object ):
     Always has a .value which is the unix timestamp as a float.  The string version is lazily produced.
     """
     UTC				= pytz.utc
-    LOC				= get_localzone() # from environment TZ, /etc/timezone, etc.
+    LOC				= get_localzone()	# from environment TZ, /etc/timezone, etc.
 
-    _precision			= 3
+    _precision			= 3			# How many default sub-second digits
+    _epsilon			= 10**-_precision	# How small a difference to consider ==
     _timeseps			= ( string
                                     if sys.version_info.major < 3
                                     else str ).maketrans( ":-.", "   " )
-    _fmt			= '%Y-%m-%d %H:%M:%S' # 2014-04-01 10:11:12
+    _fmt			= '%Y-%m-%d %H:%M:%S'	# 2014-04-01 10:11:12
 
     # A map of all the common timezone abbreviations to their canonical timezones along with the
     # proper is_dst setting.
     _tzabbrev			= {}
-    _tzabbrev_lock		= threading.Lock()
+    _cls_lock			= threading.Lock()
 
     @classmethod
-    @mutexmethod( '_tzabbrev_lock' )
+    @mutexmethod( '_cls_lock' )
     def support_abbreviations( cls, region, exclude=None, at=None, reach=None, reset=False ):
         """Add all the DST and non-DST abbreviations for the specified region.  If a country code
         (eg. 'CA') is specified, we'll get all its timezones from pytz.country_timezones.
@@ -336,7 +339,7 @@ class timestamp( object ):
         return added
 
     @classmethod
-    @mutexmethod( '_tzabbrev_lock' )
+    @mutexmethod( '_cls_lock' )
     def timezone_info( cls, tzinfo ):
         """Return the (tzinfo,is_dst) of the supplied tz. (default is_dst: None).  Accepts either a
         tzinfo, or a string and looks up the corresponding timezone abbreviation (is_dst:
@@ -399,7 +402,64 @@ class timestamp( object ):
 
     @classmethod
     def datetime_from_number( cls, n, tzinfo=None ):
-        """Convert a numeric timestamp into a datetime in the specified timezone."""
+        """Convert a numeric UNIX timestamp into a datetime in the specified timezone.  UNIX epoch times are
+        unambiguous; the target timezone's is_dst hint is not required.
+
+        TODO: When a datetime is assigned a time in a certain timezone that has DST (such as
+        America/Edmonton, MST7MDT), that specific time is either "in" our "out" of DST.  When it is
+        rendered, it will be rendered as MST or MDT.
+
+        Ideally, it would be nice to force the produced datetime to have a fixed non-DST rendering
+        such as MST, regardless of whether the time is "in" or "out" of DST.  There are certain 
+        such timezone supplied by pytz:
+
+            >>> [ z for z in pytz.all_timezones if len( z ) in (3,4) ]
+            ['CET', 'Cuba', 'EET', 'EST', 'Eire', 'GMT', 'GMT0', 'HST', 'Iran', 'MET',\
+            'MST', 'PRC', 'ROC', 'ROK', 'UCT', 'UTC', 'W-SU', 'WET', 'Zulu']
+
+        These are *different* than the 'MST' abbreviation that would be made available by invoking
+        timestamp.support_abbreviations( 'CA' ); simply the 'America/Edmonton' zone with an
+        'is_dst=False' flag!  The pytz 'MST' zone is a full pytz.tzinfo without any DST changes.
+
+            >>> ts=timestamp( 1399726367 )
+            >>> ts
+            <2014-05-10 12:52:47.000 =~= 1399726367.000000>
+            >>> ts.render( 'America/Edmonton' )
+            '2014-05-10 06:52:47.000 MDT'
+            >>> ts.render( 'MST' )
+            '2014-05-10 05:52:47.000 MST'
+
+        Now, get the "abbreviations", including 'MST'.  Note that the time is now rendered using
+        'America/Edmonton', and thus identifies the time as being in DST:
+
+            >>> timestamp.support_abbreviations( 'CA')
+            ['ADT', 'EST', 'AST', 'PDT', 'MST', 'CDT', 'PST', 'EDT', 'CST', 'NST', 'NDT', 'MDT']
+            >>> ts.render( 'MST' )
+            '2014-05-10 06:52:47.000 MDT'
+
+            >>> timestamp( '2014-05-10 05:52:47.000 MST' )
+            <2014-05-10 12:52:47.000 =~= 1399726367.000000>
+
+        To sum up:
+
+            >>> timestamp.support_abbreviations( None, reset=True )
+            []
+            >>> timestamp( '2014-05-10 05:52:47.000 MST' ).render( 'MST' )
+            '2014-05-10 05:52:47.000 MST'
+            >>> timestamp.support_abbreviations( 'CA' )
+            ['ADT', 'EST', 'AST', 'PDT', 'MST', 'CDT', 'PST', 'EDT', 'CST', 'NST', 'NDT', 'MDT']
+            >>> timestamp( '2014-05-10 05:52:47.000 MST' ).render( 'MST' )
+            '2014-05-10 05:52:47.000 MDT'
+
+        Unfortunately, the idea is simply not supported by Python datetime for the general case.
+        The best we could do would be to "denormalize" the datetime, by adding an offset to bring
+        the time "out" of DST (or by using a known non-DST time), and then adding an offset to that
+        datetime to exactly yield the target datetime, and then avoid running pytz.tzinfo.normalize
+        to "fix" the datetime's time and tzinfo.is_dst flag.  This would be a bit hacky, but might
+        be possible (see the treatise at the bottom of the Python source
+        .../Modules/datetimemodule.c for some possible pitfalls)
+
+        """
         if tzinfo is None:
             tzinfo		= cls.UTC
         elif not isinstance( tzinfo, datetime.tzinfo ):
@@ -428,7 +488,8 @@ class timestamp( object ):
         elif isinstance( value, (float, int)):
             self.value		= float( value )
         elif isinstance( value, type_str_base ):
-            self.utc		= value
+            dt			= self.datetime_from_string( value )
+            self.value		= self.number_from_datetime( dt )
         elif isinstance( value, timestamp ):
             self.value		= value.value
             self._str		= value._str
@@ -438,7 +499,7 @@ class timestamp( object ):
             raise ValueError( "Invalid timestamp of %s: %r", type( value ), value )
 
     def render( self, tzinfo=None, ms=True ):
-        """Render the time in the specified zone, optionally with milliseconds.  If the specified
+        """Render the time in the specified zone, optionally with milliseconds.  If the resultant
         timezone is not UTC, include the timezone designation in the output.
 
         Since we are "rounding" to (default) 3 places after the decimal, and since floating point
@@ -490,9 +551,9 @@ class timestamp( object ):
             > assert timestamp( 1399326141.999836 ) >= timestamp( 1399326141.374836 )
             E assert <2014-05-05 21:42:21.000 =~= 1399326141.999836> >= <2014-05-05 21:42:21.375 =~= 1399326141.374836>
 
-        So, we round to 3 places first, used the rounded timestamp to generate the datetime for
-        formatting, and then use floating point formatting to generate the rounded microseconds:
-        this produces identical results on Python 2/3:
+        So, we round to specified places of sub-second precision first, used the rounded timestamp
+        to generate the datetime for formatting, and then use floating point formatting to generate
+        the rounded microseconds: this produces identical results on Python 2/3:
 
             >>> from cpppo.history import timestamp
             >>> for v in [1414915323.122, 1414915323.123, 1414915323.124, 1414915323.125, 1414915323.126, 1414915323.127, 1399326141.999836 ]:
@@ -506,8 +567,6 @@ class timestamp( object ):
             1399326141.999835968 == 1399326142.000000000 == 1399326142.000 == 2014-05-05 21:42:22.000
 
         """
-        if tzinfo is None:
-            tzinfo		= self.UTC
         subsecond		= self._precision if ms is True else int( ms ) if ms else 0
         assert 0 <= subsecond <= 6, "Invalid sub-second precision; must be 0-6 digits"
         value			= round( self.value, subsecond ) if subsecond else self.value
@@ -515,7 +574,7 @@ class timestamp( object ):
         result			= dt.strftime( self._fmt )
         if subsecond:
             result	       += ( '%.*f' % ( subsecond, value ))[-subsecond-1:]
-        if tzinfo is not self.UTC:
+        if dt.tzinfo is not self.UTC:
             result	       += dt.strftime( ' %Z' )
         return result
 
@@ -544,7 +603,7 @@ class timestamp( object ):
         to any timezone, and produces the correct UNIX timestamp.
 
         """
-        self.value		= self.number_from_datetime( self.datetime_from_string( utctime, self.UTC ))
+        self.value		= self.number_from_datetime( self.datetime_from_string( utctime ))
         self._str		= None
 
     @property
@@ -575,24 +634,23 @@ class timestamp( object ):
         self.value		= self.number_from_datetime( self.datetime_from_string( loctime, self.LOC ))
         self._str		= None
 
-    # Comparisons.  Always lexicographically, in UTC
+    # Comparisons.  Always equivalent to lexicographically, in UTC to 3 decimal places.  However,
+    # we'll compare numerically, to avoid having to render/compare strings; if the <self>.value is
+    # within _epsilon (default: 0.001) of <rhs>.value, it is considered equal.
     def __lt__( self, rhs ):
         assert isinstance( rhs, timestamp )
-        return str( self ) < str( rhs )
-    def __le__( self, rhs ):
-        return str( self ) <= str( rhs )
+        return self.value + self.__class__._epsilon < rhs.value
     def __gt__( self, rhs ):
         assert isinstance( rhs, timestamp )
-        return str( self ) > str( rhs )
+        return self.value - self.__class__._epsilon > rhs.value
+    def __le__( self, rhs ):
+        return not self.__gt__( rhs )
     def __ge__( self, rhs ):
-        assert isinstance( rhs, timestamp )
-        return str( self ) >= str( rhs )
+        return not self.__lt__( rhs )
     def __eq__( self, rhs ):
-        assert isinstance( rhs, timestamp )
-        return str( self ) == str( rhs )
+        return not self.__ne__( rhs )
     def __ne__( self, rhs ):
-        assert isinstance( rhs, timestamp )
-        return str( self ) != str( rhs )
+        return self.__lt__( rhs ) or self.__gt__( rhs )
 
     # Add/subtract numeric seconds.  +/- 0 is a noop/copy.
     def __add__( self, rhs ):
